@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2010  Otfried Cheong
+    Copyright (C) 1993-2011  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -709,7 +709,7 @@ end
 
 function EDITTOOL:showMenu()
   local t = self:type()
-  local m = ipeui.Menu()
+  local m = ipeui.Menu(self.model.ui:win())
   local gp = self.model.ui:globalPos()
   if self:canDelete() then
     m:add("action_delete", "Delete vertex")
