@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2010  Otfried Cheong
+    Copyright (C) 1993-2011  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -36,12 +36,17 @@ require "tools"
 require "editpath"
 require "properties"
 require "shortcuts"
+require "mouse"
 
 ----------------------------------------------------------------------
 
 -- short names
 V = ipe.Vector
 
+-- store the model for the first window
+-- Used only by MacOS file_open_event
+local first_model = nil
+
 ----------------------------------------------------------------------
 
 function formatFromFileName(fname)
@@ -138,6 +143,72 @@ function transformShape(matrix, shape)
   end
 end
 
+----------------------------------------------------------------------
+-- This function is called to launch a file
+
+function file_open_event(fname)
+  if first_model and first_model.pristine then
+    first_model.loadDocument(fname)
+  else
+    MODEL:new(fname)
+  end
+end
+
+----------------------------------------------------------------------
+
+local win32_conversions = {
+  PgDown=0x22, PgUp=0x21, Home=0x24, End=0x23,
+  Left=0x25, Up=0x26, Right=0x27, Down=0x28,
+  insert=0x2d, delete=0x2e,
+  ["/"]= 0xbf, ["\\"]= 0xdc, ["-"]= 0xbd, ["="]= 0xbb,
+  ["@"]= 0x40032, -- actually Shift+2
+}
+
+function win32_shortcut_convert(s)
+  local k = 0
+  local done = false
+  while not done do
+    if s:sub(1,5) == "Ctrl+" then
+      s = s:sub(6)
+      k = k + 0x20000
+    elseif s:sub(1,6) == "Shift+" then
+      s = s:sub(7)
+      k = k + 0x40000
+    elseif s:sub(1,4) == "Alt+" then
+      s = s:sub(5)
+      k = k + 0x10000
+    else
+      done = true
+    end
+  end
+  if s:sub(1,1) == "F" and tonumber(s:sub(2)) then
+    return k + 0x6f + tonumber(s:sub(2))
+  elseif win32_conversions[s] then
+    return k + win32_conversions[s]
+  else
+    return k + string.byte(s:sub(1,1))
+  end
+end
+
+function win32_shortcuts(ui)
+  if config.toolkit ~= "win32" then return end
+  local accel = {}
+  for i in pairs(shortcuts) do
+    local id = ui:actionId(i)
+    local s = shortcuts[i]
+    if type(s) == "table" then
+      for j,s1 in ipairs(s) do
+	accel[#accel+1] = win32_shortcut_convert(s1)
+	accel[#accel+1] = id
+      end
+    elseif s then
+      accel[#accel+1] = win32_shortcut_convert(s)
+      accel[#accel+1] = id
+    end
+  end
+  return accel
+end
+
 ----------------------------------------------------------------------
 
 local function show_configuration()
@@ -221,6 +292,15 @@ for _,w in ipairs(config.ipeletDirs) do
   end
 end
 
+-- Windows provides the command line as one long string
+if config.toolkit == "win32" then
+  local t = {}
+  for w in string.gmatch(argv, "%S+") do
+    t[#t+1] = w
+  end
+  argv = t
+end
+
 if #argv == 1 and argv[1] == "-show-configuration" then
   show_configuration()
   return
@@ -256,8 +336,8 @@ for _,w in ipairs(prefs.styles) do
   config.styleList[#config.styleList + 1] = w
 end
 
-MODEL:new(first_file)
+first_model = MODEL:new(first_file)
 
-ipeui.mainLoop()
+mainloop(win32_shortcuts(first_model.ui))
 
 ----------------------------------------------------------------------
