@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2010  Otfried Cheong
+    Copyright (C) 1993-2011  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -52,7 +52,7 @@ prefs.autosave_filename = config.home .. "/%s.autosave"
 -- DOS window will pop up when calling the editor
 if os.getenv("EDITOR") then
   prefs.external_editor = os.getenv("EDITOR") .. " %s"
-elseif config.platform ~= "win" then
+elseif config.platform == "unix" then
   prefs.external_editor = "gedit %s"
   -- prefs.external_editor = "emacsclient %s"
 else
@@ -103,7 +103,9 @@ prefs.snap_distance = 16
 -- this distance, the closest object is selected instead
 prefs.close_distance = 48
 
--- Minimal and maximal possible zoom
+-- Zoom factors, minimal and maximal possible zoom
+prefs.zoom_factor = 1.3
+prefs.wheel_zoom_factor = 1.3  -- used when zooming by scroll wheel
 prefs.min_zoom = 0.1
 prefs.max_zoom = 100
 
@@ -130,6 +132,11 @@ else
   prefs.keyboard = nil
 end
 
+-- tablet mode (true or false)
+-- If tablet_mode is true, then the current selection is not highlighted
+-- when in "ink" drawing mode.
+prefs.tablet_mode = true
+
 -- format string for the coordinates in the status bar
 -- (x, unit, y, unit)
 prefs.coordinates_format = "%g%s, %g%s"
