@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2010  Otfried Cheong
+    Copyright (C) 1993-2011  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
 -- Multiple shortcuts for an action are possible, see 'delete'.
 
 shortcuts = {
+  new_window = nil,
   open = "Ctrl+O",
   quit = "Ctrl+Q",
   save = "Ctrl+S",
@@ -43,7 +44,7 @@ shortcuts = {
   paste = nil,
   paste_at_cursor = "Ctrl+V",
   cut = "Ctrl+X",
-  delete = { "Delete", "Backspace" },
+  delete = { "delete", "backspace" },
   group = "Ctrl+G",
   ungroup = "Ctrl+U",
   front = "Ctrl+F",
@@ -66,6 +67,7 @@ shortcuts = {
   mode_rotate = "R",
   mode_stretch = "E",
   mode_pan = nil,
+  mode_shredder = nil,
   mode_label = "L",
   mode_math = "Shift+4",
   mode_paragraph = "G",
@@ -103,8 +105,9 @@ shortcuts = {
   fit_width = "-",
   fit_objects = "=",
   fit_selection = "@",
-  pan_here = "x",
+  pan_here = "X",
   new_layer = "Ctrl+Shift+N",
+  rename_active_layer = "Ctrl+Shift+R",
   select_in_active_layer = "Ctrl+Shift+A",
   move_to_active_layer = "Ctrl+Shift+M",
   next_view = "PgDown",
