@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2010  Otfried Cheong
+    Copyright (C) 1993-2011  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -28,26 +28,19 @@
 
 */
 
-extern "C" {
-#include <lua.h>
-#include <lualib.h>
-#include <lauxlib.h>
-#include "lfs.h"
-}
-
-#include "ipeselector.h"
-
 #include "appui.h"
+#include "ipecanvas.h"
 #include "tools.h"
-#include "pagesorter.h"
 
 #include "ipelua.h"
 
-#include "pagesorter.h"
-
 #include <cstdio>
 #include <cstdlib>
 
+#ifdef IPEUI_QT
+#include "ipeselector_qt.h"
+#include "controls_qt.h"
+
 #include <QApplication>
 #include <QStatusBar>
 #include <QPainter>
@@ -55,22 +48,22 @@ extern "C" {
 #include <QDialog>
 #include <QVBoxLayout>
 #include <QDialogButtonBox>
+#endif
 
 using namespace ipe;
-using namespace ipeqt;
 using namespace ipelua;
 
 // --------------------------------------------------------------------
 
-static Canvas *check_canvas(lua_State *L, int i)
+static CanvasBase *check_canvas(lua_State *L, int i)
 {
-  AppUi **ui = (AppUi **) luaL_checkudata(L, i, "Ipe.appui");
+  AppUiBase **ui = (AppUiBase **) luaL_checkudata(L, i, "Ipe.appui");
   return (*ui)->canvas();
 }
 
-inline AppUi **check_appui(lua_State *L, int i)
+inline AppUiBase **check_appui(lua_State *L, int i)
 {
-  return (AppUi **) luaL_checkudata(L, i, "Ipe.appui");
+  return (AppUiBase **) luaL_checkudata(L, i, "Ipe.appui");
 }
 
 static int appui_tostring(lua_State *L)
@@ -94,7 +87,7 @@ static int appui_destructor(lua_State *L)
 
 static int appui_setPage(lua_State *L)
 {
-  AppUi **ui = check_appui(L, 1);
+  AppUiBase **ui = check_appui(L, 1);
   Page *page = check_page(L, 2)->page;
   int pno = luaL_checkinteger(L, 3) - 1;
   int view = check_viewno(L, 4, page);
@@ -105,7 +98,7 @@ static int appui_setPage(lua_State *L)
 
 static int appui_setFontPool(lua_State *L)
 {
-  Canvas *canvas = check_canvas(L, 1);
+  CanvasBase *canvas = check_canvas(L, 1);
   Document **d = check_document(L, 2);
   const FontPool *fonts = (*d)->fontPool();
   canvas->setFontPool(fonts);
@@ -114,7 +107,7 @@ static int appui_setFontPool(lua_State *L)
 
 static int appui_pan(lua_State *L)
 {
-  Canvas *canvas = check_canvas(L, 1);
+  CanvasBase *canvas = check_canvas(L, 1);
   Vector p = canvas->pan();
   push_vector(L, p);
   return 1;
@@ -122,7 +115,7 @@ static int appui_pan(lua_State *L)
 
 static int appui_setPan(lua_State *L)
 {
-  Canvas *canvas = check_canvas(L, 1);
+  CanvasBase *canvas = check_canvas(L, 1);
   Vector *v = check_vector(L, 2);
   canvas->setPan(*v);
   return 0;
@@ -130,53 +123,60 @@ static int appui_setPan(lua_State *L)
 
 static int appui_zoom(lua_State *L)
 {
-  Canvas *canvas = check_canvas(L, 1);
+  CanvasBase *canvas = check_canvas(L, 1);
   lua_pushnumber(L, canvas->zoom());
   return 1;
 }
 
 static int appui_setZoom(lua_State *L)
 {
-  AppUi **ui = check_appui(L, 1);
+  AppUiBase **ui = check_appui(L, 1);
   (*ui)->setZoom(luaL_checknumber(L, 2));
   return 0;
 }
 
 static int appui_pos(lua_State *L)
 {
-  Canvas *canvas = check_canvas(L, 1);
+  CanvasBase *canvas = check_canvas(L, 1);
   push_vector(L, canvas->pos());
   return 1;
 }
 
 static int appui_globalPos(lua_State *L)
 {
-  Canvas *canvas = check_canvas(L, 1);
+  CanvasBase *canvas = check_canvas(L, 1);
   push_vector(L, canvas->globalPos());
   return 1;
 }
 
 static int appui_unsnappedPos(lua_State *L)
 {
-  Canvas *canvas = check_canvas(L, 1);
+  CanvasBase *canvas = check_canvas(L, 1);
   push_vector(L, canvas->unsnappedPos());
   return 1;
 }
 
 static int appui_simpleSnapPos(lua_State *L)
 {
-  Canvas *canvas = check_canvas(L, 1);
+  CanvasBase *canvas = check_canvas(L, 1);
   push_vector(L, canvas->simpleSnapPos());
   return 1;
 }
 
 static int appui_setFifiVisible(lua_State *L)
 {
-  Canvas *canvas = check_canvas(L, 1);
+  CanvasBase *canvas = check_canvas(L, 1);
   canvas->setFifiVisible(lua_toboolean(L, 2));
   return 0;
 }
 
+static int appui_setSelectionVisible(lua_State *L)
+{
+  CanvasBase *canvas = check_canvas(L, 1);
+  canvas->setSelectionVisible(lua_toboolean(L, 2));
+  return 0;
+}
+
 static void snapFlag(lua_State *L, int &flags, const char *mode, uint bits)
 {
   lua_getfield(L, -1, mode);
@@ -187,7 +187,7 @@ static void snapFlag(lua_State *L, int &flags, const char *mode, uint bits)
 
 static int appui_setSnap(lua_State *L)
 {
-  Canvas *canvas = check_canvas(L, 1);
+  CanvasBase *canvas = check_canvas(L, 1);
   luaL_checktype(L, 2, LUA_TTABLE);
   Snap snap = canvas->snap();
   snapFlag(L, snap.iSnap, "snapvtx", Snap::ESnapVtx);
@@ -230,7 +230,7 @@ static int appui_setSnap(lua_State *L)
 
 static int appui_setAutoOrigin(lua_State *L)
 {
-  Canvas *canvas = check_canvas(L, 1);
+  CanvasBase *canvas = check_canvas(L, 1);
   Vector *v = check_vector(L, 2);
   canvas->setAutoOrigin(*v);
   return 0;
@@ -238,7 +238,7 @@ static int appui_setAutoOrigin(lua_State *L)
 
 static int appui_update(lua_State *L)
 {
-  Canvas *canvas = check_canvas(L, 1);
+  CanvasBase *canvas = check_canvas(L, 1);
   bool all = true;
   if (lua_gettop(L) == 2)
     all = lua_toboolean(L, 2);
@@ -251,15 +251,15 @@ static int appui_update(lua_State *L)
 
 static int appui_finishTool(lua_State *L)
 {
-  Canvas *canvas = check_canvas(L, 1);
+  CanvasBase *canvas = check_canvas(L, 1);
   canvas->finishTool();
   return 0;
 }
 
 static int appui_canvasSize(lua_State *L)
 {
-  Canvas *canvas = check_canvas(L, 1);
-  push_vector(L, Vector(canvas->width(), canvas->height()));
+  CanvasBase *canvas = (CanvasBase *) check_canvas(L, 1);
+  push_vector(L, Vector(canvas->canvasWidth(), canvas->canvasHeight()));
   return 1;
 }
 
@@ -276,38 +276,29 @@ static void check_rgb(lua_State *L, int i, int &r, int &g, int &b)
 
 static int appui_setCursor(lua_State *L)
 {
-  Canvas *canvas = check_canvas(L, 1);
+  CanvasBase *canvas = check_canvas(L, 1);
   if (lua_isnumber(L, 2)) {
     double s = lua_tonumber(L, 2);
     int r, g, b;
     check_rgb(L, 3, r, g, b);
-    QPixmap p(32, 32);
-    p.fill(QColor(255, 255, 255, 0));
-    QPainter painter(&p);
-    s = 0.5 * s * canvas->zoom();
-    if (s < 1.0)
-      s = 1.0;
-    if (s > 10.0)
-      s = 10.0;
-    r = 255 * r / 1000;
-    g = 255 * g / 1000;
-    b = 255 * b / 1000;
-    painter.setBrush(QColor(r, g, b));
-    painter.setPen(Qt::NoPen);
-    painter.drawEllipse(QRectF(16.0 - s, 16.0 - s, 2 * s, 2 * s));
-    painter.end();
-    QCursor cursor(p);
-    canvas->setCursor(cursor);
+    Color color(r, g, b);
+    canvas->setCursor(CanvasBase::EDotCursor, s, &color);
+  } else if (lua_isstring(L, 2)) {
+    static const char * const cursor_names[] =
+      { "standard", "hand", "cross", 0 };
+    CanvasBase::TCursor t =
+      CanvasBase::TCursor(luaL_checkoption(L, 2, 0, cursor_names));
+    canvas->setCursor(t);
   } else
-    canvas->unsetCursor();
+    canvas->setCursor(CanvasBase::EStandardCursor);
   return 0;
 }
 
 static int appui_setNumbering(lua_State *L)
 {
-  Canvas *canvas = check_canvas(L, 1);
+  CanvasBase *canvas = check_canvas(L, 1);
   bool t = lua_toboolean(L, 2);
-  Canvas::Style s = canvas->canvasStyle();
+  CanvasBase::Style s = canvas->canvasStyle();
   s.numberPages = t;
   canvas->setCanvasStyle(s);
   return 0;
@@ -317,7 +308,7 @@ static int appui_setNumbering(lua_State *L)
 
 static int appui_pantool(lua_State *L)
 {
-  Canvas *canvas = check_canvas(L, 1);
+  CanvasBase *canvas = check_canvas(L, 1);
   Page *page = check_page(L, 2)->page;
   int view = check_viewno(L, 3, page);
   PanTool *tool = new PanTool(canvas, page, view);
@@ -327,7 +318,7 @@ static int appui_pantool(lua_State *L)
 
 static int appui_selecttool(lua_State *L)
 {
-  Canvas *canvas = check_canvas(L, 1);
+  CanvasBase *canvas = check_canvas(L, 1);
   Page *page = check_page(L, 2)->page;
   int view = check_viewno(L, 3, page);
   double selectDistance = luaL_checknumber(L, 4);
@@ -342,7 +333,7 @@ static int appui_transformtool(lua_State *L)
 {
   static const char * const option_names[] = {
     "translate", "scale", "stretch", "rotate", 0 };
-  Canvas *canvas = check_canvas(L, 1);
+  CanvasBase *canvas = check_canvas(L, 1);
   Page *page = check_page(L, 2)->page;
   int view = check_viewno(L, 3, page);
   int type = luaL_checkoption(L, 4, 0, option_names);
@@ -414,7 +405,7 @@ static int pastetool_setmatrix(lua_State *L)
 
 static int appui_shapetool(lua_State *L)
 {
-  Canvas *canvas = check_canvas(L, 1);
+  CanvasBase *canvas = check_canvas(L, 1);
   lua_pushvalue(L, 2);
   int luatool = luaL_ref(L, LUA_REGISTRYINDEX);
   ShapeTool *tool = new ShapeTool(canvas, L, luatool);
@@ -435,7 +426,7 @@ static int appui_shapetool(lua_State *L)
 
 static int appui_pastetool(lua_State *L)
 {
-  Canvas *canvas = check_canvas(L, 1);
+  CanvasBase *canvas = check_canvas(L, 1);
   Object *obj = check_object(L, 2)->obj;
   lua_pushvalue(L, 3);
   int luatool = luaL_ref(L, LUA_REGISTRYINDEX);
@@ -454,39 +445,48 @@ static int appui_pastetool(lua_State *L)
 
 // --------------------------------------------------------------------
 
+static int appui_win(lua_State *L)
+{
+  AppUiBase **ui = check_appui(L, 1);
+  push_winid(L, (*ui)->windowId());
+  return 1;
+}
+
 static int appui_close(lua_State *L)
 {
-  AppUi **ui = check_appui(L, 1);
-  lua_pushnumber(L, (*ui)->close());
+  AppUiBase **ui = check_appui(L, 1);
+  (*ui)->closeWindow();
+  return 0;
+}
+
+static int appui_actionId(lua_State *L)
+{
+  AppUiBase **ui = check_appui(L, 1);
+  const char *action = luaL_checkstring(L, 2);
+  lua_pushinteger(L, (*ui)->actionId(action));
   return 1;
 }
 
 static int appui_actionState(lua_State *L)
 {
-  AppUi **ui = check_appui(L, 1);
+  AppUiBase **ui = check_appui(L, 1);
   const char *name = luaL_checkstring(L, 2);
-  QAction *a = (*ui)->findAction(name);
-  if (a) {
-    lua_pushboolean(L, a->isChecked());
-    return 1;
-  } else
-    return 0;
+  lua_pushboolean(L, (*ui)->actionState(name));
+  return 1;
 }
 
 static int appui_setActionState(lua_State *L)
 {
-  AppUi **ui = check_appui(L, 1);
+  AppUiBase **ui = check_appui(L, 1);
   const char *name = luaL_checkstring(L, 2);
   bool val = lua_toboolean(L, 3);
-  QAction *a = (*ui)->findAction(name);
-  if (a)
-    a->setChecked(val);
+  (*ui)->setActionState(name, val);
   return 0;
 }
 
 static int appui_setupSymbolicNames(lua_State *L)
 {
-  AppUi **ui = check_appui(L, 1);
+  AppUiBase **ui = check_appui(L, 1);
   Cascade *sheets = check_cascade(L, 2)->cascade;
   (*ui)->setupSymbolicNames(sheets);
   return 0;
@@ -494,7 +494,7 @@ static int appui_setupSymbolicNames(lua_State *L)
 
 static int appui_setAttributes(lua_State *L)
 {
-  AppUi **ui = check_appui(L, 1);
+  AppUiBase **ui = check_appui(L, 1);
   Cascade *sheets = check_cascade(L, 2)->cascade;
   AllAttributes all;
   check_allattributes(L, 3, all);
@@ -504,7 +504,7 @@ static int appui_setAttributes(lua_State *L)
 
 static int appui_setGridAngleSize(lua_State *L)
 {
-  AppUi **ui = check_appui(L, 1);
+  AppUiBase **ui = check_appui(L, 1);
   Attribute gs = check_number_attribute(L, 2);
   Attribute as = check_number_attribute(L, 3);
   (*ui)->setGridAngleSize(gs, as);
@@ -513,7 +513,7 @@ static int appui_setGridAngleSize(lua_State *L)
 
 static int appui_setLayers(lua_State *L)
 {
-  AppUi **ui = check_appui(L, 1);
+  AppUiBase **ui = check_appui(L, 1);
   Page *page = check_page(L, 2)->page;
   int view = check_viewno(L, 3, page);
   (*ui)->setLayers(page, view);
@@ -522,7 +522,7 @@ static int appui_setLayers(lua_State *L)
 
 static int appui_setNumbers(lua_State *L)
 {
-  AppUi **ui = check_appui(L, 1);
+  AppUiBase **ui = check_appui(L, 1);
   String vno;
   String pno;
   if (!lua_isnil(L, 2))
@@ -537,21 +537,32 @@ static int appui_setNumbers(lua_State *L)
 
 static int appui_setBookmarks(lua_State *L)
 {
-  AppUi **ui = check_appui(L, 1);
-  return (*ui)->setBookmarks(L);
+  AppUiBase **ui = check_appui(L, 1);
+  luaL_argcheck(L, lua_istable(L, 2), 2, "argument is not a table");
+  int no = lua_objlen(L, 2);
+  String bm[no];
+  for (int i = 1; i <= no; ++i) {
+    lua_rawgeti(L, 2, i);
+    luaL_argcheck(L, lua_isstring(L, -1), 2, "item is not a string");
+    bm[i-1] = String(lua_tostring(L, -1));
+    lua_pop(L, 1);
+  }
+  (*ui)->setBookmarks(no, bm);
+  return 0;
 }
 
 static int appui_setWindowTitle(lua_State *L)
 {
-  AppUi **ui = check_appui(L, 1);
-  const char *s = luaL_checkstring(L, 2);
-  (*ui)->setWindowTitle(QString::fromUtf8(s));
+  AppUiBase **ui = check_appui(L, 1);
+  bool mod = lua_toboolean(L, 2);
+  const char *s = luaL_checkstring(L, 3);
+  (*ui)->setWindowCaption(mod, s);
   return 0;
 }
 
 static int appui_setNotes(lua_State *L)
 {
-  AppUi **ui = check_appui(L, 1);
+  AppUiBase **ui = check_appui(L, 1);
   const char *s = luaL_checkstring(L, 2);
   (*ui)->setNotes(s);
   return 0;
@@ -559,30 +570,30 @@ static int appui_setNotes(lua_State *L)
 
 static int appui_showTool(lua_State *L)
 {
-  AppUi **ui = check_appui(L, 1);
-  return (*ui)->showTool(L);
+  static const char * const option_names[] = {
+    "properties", "bookmarks", "notes", "layers", 0 };
+
+  AppUiBase **ui = check_appui(L, 1);
+  int m = luaL_checkoption(L, 2, 0, option_names);
+  bool s = lua_toboolean(L, 3);
+  (*ui)->setToolVisible(m, s);
+  return 0;
 }
 
 static int appui_explain(lua_State *L)
 {
-  AppUi **ui = check_appui(L, 1);
+  AppUiBase **ui = check_appui(L, 1);
   const char *s = luaL_checkstring(L, 2);
   int t = 4000;
   if (lua_isnumber(L, 3))
     t = lua_tointeger(L, 3);
-  (*ui)->statusBar()->showMessage(QString::fromUtf8(s), t);
+  (*ui)->explain(s, t);
   return 0;
 }
 
-static int appui_checkbox(lua_State *L)
-{
-  AppUi **ui = check_appui(L, 1);
-  lua_pushboolean(L, (*ui)->checkbox(L));
-  return 1;
-}
-
 // --------------------------------------------------------------------
 
+#if IPEUI_QT
 static void get_page_sorter_size(lua_State *L, int &width, int &height,
 				 int &thumbWidth)
 {
@@ -608,10 +619,12 @@ static void get_page_sorter_size(lua_State *L, int &width, int &height,
     thumbWidth = lua_tointeger(L, -1);
   lua_pop(L, 1); // thumbnail_width, prefs
 }
+#endif
 
 static int appui_selectPage(lua_State *L)
 {
-  AppUi **ui = check_appui(L, 1);
+#if IPEUI_QT
+  AppUiBase **ui = check_appui(L, 1);
   (void) ui;  // not actually used now
   Document **doc = check_document(L, 2);
   int page = -1;
@@ -639,12 +652,14 @@ static int appui_selectPage(lua_State *L)
     lua_pushinteger(L, sel + 1);
     return 1;
   } else
+#endif
     return 0;
 }
 
 static int appui_pageSorter(lua_State *L)
 {
-  AppUi **ui = check_appui(L, 1);
+#ifdef IPEUI_QT
+  AppUiBase **ui = check_appui(L, 1);
   (void) ui;  // not actually used now
   Document **doc = check_document(L, 2);
 
@@ -679,6 +694,9 @@ static int appui_pageSorter(lua_State *L)
   }
   delete d;
   return 1;
+#else
+  return 0;
+#endif
 }
 
 // --------------------------------------------------------------------
@@ -698,6 +716,7 @@ static const struct luaL_Reg appui_methods[] = {
   { "unsnappedPos", appui_unsnappedPos},
   { "simpleSnapPos", appui_simpleSnapPos },
   { "setFifiVisible", appui_setFifiVisible},
+  { "setSelectionVisible", appui_setSelectionVisible},
   { "setSnap", appui_setSnap},
   { "setAutoOrigin", appui_setAutoOrigin },
   { "update", appui_update},
@@ -712,11 +731,12 @@ static const struct luaL_Reg appui_methods[] = {
   { "shapeTool", appui_shapetool },
   { "pasteTool", appui_pastetool },
   // --------------------------------------------------------------------
+  { "win", appui_win},
   { "close", appui_close},
   { "setActionState", appui_setActionState },
   { "actionState", appui_actionState },
+  { "actionId", appui_actionId },
   { "explain", appui_explain },
-  { "checkbox", appui_checkbox },
   { "setWindowTitle", appui_setWindowTitle },
   { "setupSymbolicNames", appui_setupSymbolicNames },
   { "setAttributes", appui_setAttributes },
@@ -737,16 +757,16 @@ static int appui_constructor(lua_State *L)
 {
   luaL_checktype(L, 1, LUA_TTABLE); // this is the model
 
-  AppUi **ui = (AppUi **) lua_newuserdata(L, sizeof(AppUi *));
+  AppUiBase **ui = (AppUiBase **) lua_newuserdata(L, sizeof(AppUiBase *));
   *ui = 0;
   luaL_getmetatable(L, "Ipe.appui");
   lua_setmetatable(L, -2);
 
   lua_pushvalue(L, 1);
   int model = luaL_ref(L, LUA_REGISTRYINDEX);
-  *ui = new AppUi(L, model);
+  *ui = createAppUi(L, model);
 
-  Canvas::Style style;
+  CanvasBase::Style style;
   style.pretty = false;
   style.paperColor = Color(1000,1000,1000);
   style.classicGrid = false;
@@ -809,10 +829,8 @@ static int appui_constructor(lua_State *L)
 
   (*ui)->canvas()->setCanvasStyle(style);
 
-  if (width > 0 && height > 0)
-    (*ui)->resize(width, height);
+  (*ui)->showWindow(width, height);
 
-  (*ui)->show();
   return 1;
 }
 
