@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2010  Otfried Cheong
+    Copyright (C) 1993-2011  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -88,11 +88,11 @@ end
 
 function select_symbol(model)
   local s = model.doc:sheets():allNames("symbol")
-  local d = ipeui.Dialog(model.ui, "Use symbol")
+  local d = ipeui.Dialog(model.ui:win(), "Use symbol")
   d:add("label", "label", { label = "Select symbol" }, 1, 1, 1, 3)
   d:add("select", "combo", s, 2, 1, 1, 3)
-  d:add("ok", "button", { label="&Ok", action="accept" }, 3, 3)
-  d:add("cancel", "button", { label="&Cancel", action="reject" }, 3, 2)
+  d:addButton("cancel", "&Cancel", "reject")
+  d:addButton("ok", "&Ok", "accept")
   d:setStretch("column", 1, 1)
   if not d:execute() then return end
   return s[d:get("select")]
@@ -129,7 +129,7 @@ function create_symbol(model, num)
   local p = model:page()
   local prim = p:primarySelection()
   if not prim then model.ui:explain("no selection") return end
-  local str = ipeui.getString(model.ui, "Enter name of new symbol")
+  local str = model:getString("Enter name of new symbol")
   if not str or str:match("^%s*$") then return end
   local name = str:match("^%s*%S+%s*$")
   local old = model.doc:sheets():find("symbol", name)
