@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2010  Otfried Cheong
+    Copyright (C) 1993-2011  Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -568,9 +568,7 @@ String Page::section(int level) const
 
   If \a useTitle is \c true, then \a name is ignored, and the section
   title will be copied from the page title (and further changes to the
-  page title are automatically reflected.
-
-  This function marks the page as edited. */
+  page title are automatically reflected). */
 void Page::setSection(int level, bool useTitle, String name)
 {
   iUseTitle[level] = useTitle;
